# ipdac.tcl---IP-DAC IP pack diagnostic frame
# 

set debug 0
set started 0

set testmode 0
set monitor 0

set frame ipdac
set frametitle "IP-DAC Diagnostics Frame"
set icontitle "ipdac"

if {[info exists env(VmeServer)]} then {set resource $env(VmeServer)} \
    else {set resource VME}

set mname "IPDAC"

set VIPC616BaseAddress 0xffffc000
set IPslot 0

loadsource ipcommon

proc define-module {} {
  global resource mname IPslot VIPC616BaseAddress
  set-footer "deleting all IP-DAC registers"
  EG "register :pattern '/$resource/$mname.*' !delete"

#     ID
      define-register xIP ManID        "-a $VIPC616BaseAddress -s $IPslot -o 0x09 -d id -p ro -w 8 -zx"
      define-register xIP ModID        "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d id -p ro -w 8 -zx"

#    DAC 1
      define-register xIP c1.DAC        "-a $VIPC616BaseAddress -s $IPslot -o 0x04 -d io -p wo -w 16 -zx"
      define-register xIP c1.offset     "-a $VIPC616BaseAddress -s $IPslot -o 0x19 -d id -p ro -w 8 -zx"
      define-register xIP c1.gain       "-a $VIPC616BaseAddress -s $IPslot -o 0x25 -d id -p ro -w 8 -zx"

#    DAC 2
      define-register xIP c2.DAC        "-a $VIPC616BaseAddress -s $IPslot -o 0x02 -d io -p wo -w 16 -zx"
      define-register xIP c2.offset     "-a $VIPC616BaseAddress -s $IPslot -o 0x1b -d id -p ro -w 8 -zx"
      define-register xIP c2.gain       "-a $VIPC616BaseAddress -s $IPslot -o 0x27 -d id -p ro -w 8 -zx"

#    DAC 3
      define-register xIP c3.DAC        "-a $VIPC616BaseAddress -s $IPslot -o 0x0c -d io -p wo -w 16 -zx"
      define-register xIP c3.offset     "-a $VIPC616BaseAddress -s $IPslot -o 0x1d -d id -p ro -w 8 -zx"
      define-register xIP c3.gain       "-a $VIPC616BaseAddress -s $IPslot -o 0x29 -d id -p ro -w 8 -zx"

#    DAC 4
      define-register xIP c4.DAC        "-a $VIPC616BaseAddress -s $IPslot -o 0x0a -d io -p wo -w 16 -zx"
      define-register xIP c4.offset     "-a $VIPC616BaseAddress -s $IPslot -o 0x1f -d id -p ro -w 8 -zx"
      define-register xIP c4.gain       "-a $VIPC616BaseAddress -s $IPslot -o 0x2b -d id -p ro -w 8 -zx"

#    DAC 5
      define-register xIP c5.DAC        "-a $VIPC616BaseAddress -s $IPslot -o 0x14 -d io -p wo -w 16 -zx"
      define-register xIP c5.offset     "-a $VIPC616BaseAddress -s $IPslot -o 0x21 -d id -p ro -w 8 -zx"
      define-register xIP c5.gain       "-a $VIPC616BaseAddress -s $IPslot -o 0x2d -d id -p ro -w 8 -zx"

#    DAC 6
      define-register xIP c6.DAC        "-a $VIPC616BaseAddress -s $IPslot -o 0x12 -d io -p wo -w 16 -zx"
      define-register xIP c6.offset     "-a $VIPC616BaseAddress -s $IPslot -o 0x19 -d id -p ro -w 8 -zx"
      define-register xIP c6.gain       "-a $VIPC616BaseAddress -s $IPslot -o 0x2f -d id -p ro -w 8 -zx"

     click-redisplay
}

proc update-widget {widget value} {
   :modify $widget :value $value
}

proc click-dac {w v} {
    write-register c[string range $w 3 3].DAC $v
}

proc click-redisplay {} {

     set manid [read-register ManID]
     update-widget MANID $manid

     set modid [read-register ModID]
     update-widget MODID $modid

     if {$manid == 0xf0 && $modid == 0x16} then {set passed 1} else {set passed 0}
     update-widget PASSED $passed

      for {set i 1} {$i <= 6} {incr i} {
          update-widget DAC$i  [read-register c$i.DAC]
          set offset [read-register c$i.offset]
          set gain [read-register c$i.gain]
          insert-log "IP-DAC: Channel $i -  offset $offset, gain $gain"
      }
}

proc click-reset {} {}

proc click arg {
   global debug
  global VIPC616BaseAddress IPslot

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      IPBASEADDR   {
          set VIPC616BaseAddress $v
          define-module
      }
      IPSLOTADDR {
          set IPslot $v
          define-module
      }
      DAC*  {click-dac $w $v}

      default   {puts stdout $arg}
   }
}



proc make-frame {} {
  global env VIPC616BaseAddress IPslot

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 5

      :text IPBASEADDR   :label "IP Carrier Base Address:" :value $VIPC616BaseAddress :w 10
      :choice IPSLOTADDR :label "IP Pack slot" :strings [list A B C D E F]         :value $IPslot
      :next-row T; :col-gap 20
      :text MANID        :label "Man ID"  :w 4
      :text MODID        :label "Mod ID"  :w 4
      :checkbox PASSED       :label "IP check passed"

   :panel CLICK :layout T :padx 50 :pady 10
      :layout V; :row-gap 5; :col-gap 15

      for {set i 1} {$i <= 6} {incr i} {
          :slider DAC$i  :label "DAC $i"  :min 0 :max 4095
      }
   
   :panel TEST  :layout T :padx 10 :pady 10
      :layout H; :col-gap 15
      :button REDISPLAY :label "Redisplay"              :command "click-redisplay"
      :button RESET     :label "Reset"                  :command click-reset
      :button DEFINE    :label "Define IP-DAC registers" :command "define-module"
   
   :show T

}


proc start args {
   global started
   if {$started == 0} then {
      make-frame
      set started 1
#      define-module
#      click-reset
   } else {open-frame}
}

proc Start {s} {
    global IPslot started

    set IPslot $s
    set started 0
    start
}
