# ipunidig-e-48.tcl---IP-Unidig-E-48 IP pack diagnostic frame
# 

set debug 0
set started 0

set testmode 0
set monitor 0

set frame ipdige48
set frametitle "IP-Unidig-E-48 Diagnostics Frame"
set icontitle "ipdige48"

if {[info exists env(VmeServer)]} then {set resource $env(VmeServer)} \
    else {set resource VME}

set mname "IPDIGE48"

set VIPC616BaseAddress 0xffffc000
set IPslot 3

loadsource ipcommon


proc define-module {} {
  global resource mname IPslot VIPC616BaseAddress
  set-footer "deleting all IP-DIGE48 registers"
  EG "register :pattern '/$resource/$mname.*' !delete"

#     ID
      define-register xIP ManID        "-a $VIPC616BaseAddress -s $IPslot -o 0x09 -d id -p ro -w 8 -zx"
      define-register xIP ModID        "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d id -p ro -w 8 -zx"

#     Output
      define-register xIP Output_1-16  "-a $VIPC616BaseAddress -s $IPslot -o 0x00 -d io -p rw -w 16 -zx"
      define-register xIP Output_17-24 "-a $VIPC616BaseAddress -s $IPslot -o 0x02 -d io -p rw -w 16 -zx"
      define-register xIP Output_25-40 "-a $VIPC616BaseAddress -s $IPslot -o 0x40 -d io -p rw -w 16 -zx"
      define-register xIP Output_41-48 "-a $VIPC616BaseAddress -s $IPslot -o 0x42 -d io -p rw -w 16 -zx"

#     Input
      define-register xIP Input_1-16  "-a $VIPC616BaseAddress -s $IPslot -o 0x04 -d io -p ro -w 16 -zx"
      define-register xIP Input_17-24 "-a $VIPC616BaseAddress -s $IPslot -o 0x06 -d io -p ro -w 16 -zx"
      define-register xIP Input_25-40 "-a $VIPC616BaseAddress -s $IPslot -o 0x44 -d io -p ro -w 16 -zx"
      define-register xIP Input_41-48 "-a $VIPC616BaseAddress -s $IPslot -o 0x46 -d io -p ro -w 16 -zx"

#    Update IP details

     set manid [read-register ManID]
     update-widget MANID $manid

     set modid [read-register ModID]
     update-widget MODID $modid

     if {$manid == 0xf0 && $modid == 0x65} then {set passed 1} else {set passed 0}
     update-widget PASSED $passed

     click-redisplay
}

proc update-widget {widget value} {
   :modify $widget :value $value
}

proc click-redisplay {} {

#    Update input line states
      set dataword [read-register Input_1-16]
      for {set i 0} {$i < 16} {incr i} {
          set bit [expr ($dataword >> $i) & 1]
          update-widget INPUT[expr $i+1]  $bit
      }
      set dataword [read-register Input_17-24]
      for {set i 0} {$i < 8} {incr i} {
          set bit [expr ($dataword >> $i) & 1]
          update-widget INPUT[expr $i+17]  $bit
      }
      set dataword [read-register Input_25-40]
      for {set i 0} {$i < 16} {incr i} {
          set bit [expr ($dataword >> $i) & 1]
          update-widget INPUT[expr $i+25]  $bit
      }
      set dataword [read-register Input_41-48]
      for {set i 0} {$i < 8} {incr i} {
          set bit [expr ($dataword >> $i) & 1]
          update-widget INPUT[expr $i+41]  $bit
      }

#    Update output line states
      set dataword [read-register Output_1-16]
      for {set i 0} {$i < 16} {incr i} {
          set bit [expr ($dataword >> $i) & 1]
          update-widget OUTPUT[expr $i+1]  $bit
      }
      set dataword [read-register Output_17-24]
      for {set i 0} {$i < 8} {incr i} {
          set bit [expr ($dataword >> $i) & 1]
          update-widget OUTPUT[expr $i+17]  $bit
      }
      set dataword [read-register Output_25-40]
      for {set i 0} {$i < 16} {incr i} {
          set bit [expr ($dataword >> $i) & 1]
          update-widget OUTPUT[expr $i+25]  $bit
      }
      set dataword [read-register Output_41-48]
      for {set i 0} {$i < 8} {incr i} {
          set bit [expr ($dataword >> $i) & 1]
          update-widget OUTPUT[expr $i+41]  $bit
      }

}

proc click-output {w v} {

   set line [string range $w 6 end]
   if  {$line <= 16} then {set f 1; set r Output_1-16}  \
   elseif  {$line <= 24} then {set f 17; set r Output_17-24}  \
   elseif  {$line <= 40} then {set f 25; set r Output_25-40}  \
   else {set f 41; set r Output_41-48}

    set bit [expr 1 << ($line - $f)]
    set dataword [read-register $r]
    switch $v {
        0    {set outputword [expr ($dataword | $bit) ^ $bit]}
        1    {set outputword [expr $dataword | $bit]}
    }
    write-register $r $outputword
}

proc click-reset {args} {}

proc click arg {
   global debug
  global VIPC616BaseAddress IPslot

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      IPBASEADDR   {
          set VIPC616BaseAddress $v
          define-module
      }
      IPSLOTADDR {
          set IPslot $v
          define-module
      }
      OUTPUT*  {click-output $w $v; click-redisplay}

      default   {puts stdout $arg}
   }
}


proc make-frame {} {
  global env VIPC616BaseAddress IPslot

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 20

      :text IPBASEADDR   :label "IP Carrier Base Address:" :value $VIPC616BaseAddress :w 12
      :choice IPSLOTADDR :label "IP Pack slot" :strings [list A B C D]         :value $IPslot
      :text MANID        :label "Man ID"  :w 4
      :text MODID        :label "Mod ID"  :w 4
      :checkbox PASSED       :label "IP check passed"

   :panel CLICK :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 15

      :message MO  :label "Output Lines" :bold T
      :next-row T

      for {set i 1} {$i <= 48} {incr i} {
          :checkbox OUTPUT$i  :label "$i"  :valuex 3c
          if {[expr ($i / 16) * 16] == $i} {:next-row T}
      }

      :message MI  :label "Input Lines" :bold T
      :next-row T

      for {set i 1} {$i <= 48} {incr i} {
          :checkbox INPUT$i   :label "$i"  :valuex 3c :deaf T
          if {[expr ($i / 16) * 16] == $i} {:next-row T}
      }
   
   :panel TEST  :layout T :padx 10 :pady 10
      :layout H; :col-gap 15
      :button REDISPLAY :label "Redisplay"              :command click-redisplay
      :button RESET     :label "Reset"                  :command click-reset
      :button DEFINE    :label "Define IP-Digital48 registers" :command define-module
   
   :show T
}


proc start args {
   global started
   if {$started == 0} then {
      make-frame
      set started 1
#      define-module
#      click-reset
   } else {open-frame}
}




#proc read-register  {r}   {global $r; return [set $r]}
#proc write-register {r v} {global $r; set $r $v}
