# V1724.tcl---CAEN V1724 module control frame



set frame "V1724"
set frametitle "CAEN V1724 Setup & Monitor"
set icontitle "V1724"

set modulename v1724
set moduletype  "CAEN V1724"

set DPPEnable 1

loadsource common.vme

set allmodules 0

set electronicsvisible 0
set electronicsframe "[self frame-name].electronics"

set savedirectory CAENtdcParameters


set expertlist {                                                    \
   {"Reset Board"                        "do_write_registers Clear 0"}          \
   {"Start Acquisition"                  "do_write_registers AcqControl 4"}       \
   {"Stop Acquisition"                   "do_write_registers AcqControl 0"}       \

}

proc midas-information {text} {

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :transient T

    :galley G :deaf T :strings $text :font fixed :rows 50
    :show T
}

proc do_read_register {r} {
    global modulename module
    return [read-vme-register $modulename#$module.$r]
}

proc do_write_register {r vl} {
    global modulename module
    return [write-vme-register $modulename#$module.$r $vl]
}

proc do_write_registers {r vl} {
    global modulename module
    global allmodules

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$r $vl
    } else {
        write-vme-registers $modulename#*.$r $vl
    }
}

proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc click-electronics {} {
#  display electronics sub frame
   global electronicsvisible electronicsframe
   if {$electronicsvisible==0} then {make-electronics} else {open-frame $electronicsframe}
   click-redisplay
}

proc click-registers {} {
   global EGreport moduletype modulename module registers_table

   if {[info procs [set moduletype]_registers] == ""} then  \
        {set-footer "No [set moduletype]_registers procedure"; return}

   [set moduletype]_registers

   clear-footer
   set EGreport -1

   set info "[set moduletype] - module $modulename#$module\n\n"

   foreach item $registers_table {
       set description [first $item]
       set register [second $item]
       set data [read-vme-register $modulename#$module.$register]
       append info "[format "%-40s %-20s" $description $data]\n"
   }

  midas-information $info

   set EGreport 0
}

proc click-select-module {v} {
   global debug module maxmodule
   if {$debug} then {puts stderr "click-select-module with argument \{$v\}"}
   set module $v
   if {$module<1} then {set module 1}
   if {$module>$maxmodule} then {set module $maxmodule}
   click-redisplay
}

proc click-all-modules {v} {global allmodules; set allmodules $v}

proc repaint {} {}

proc redisplay-module {} {
   global modulename module maxmodule allmodules

   set maxmodule [inform vmecontrol inquire [set modulename]count]
   set menu {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend menu $i}
   :modify MODULE-SEL :strings $menu
   :modify MODULE-NUM :max $maxmodule
   :modify MODULE-NUM :value $module
   :modify ALL-MODULES :value $allmodules

}


proc redisplay-electronics {} {
   global electronicsvisible
   global modulename module 

   if {$electronicsvisible==0} then {return}
}

proc click-redisplay {} {
   global EGreport

   clear-footer
   set EGreport -1
   set-busy

   redisplay-module
   redisplay-electronics

   clear-busy
   set EGreport 0
}


proc click arg {
   global debug expertlist adjset
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM  {click-select-module $v}

      ALL-MODULES {click-all-modules $v}

      ENGINEERING       {click-$v}
      REDISPLAY  	{click-redisplay}

      EXPERT  {
         set action [lindex $expertlist $i]
         set r [midas-confirmation "You are about to execute the action \"[first $action]\"\n\nAre you sure?"]
         if {$r == 1} then {eval [second $action]}
      }

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p [list $v $i]} else {insert-debug "$arg"}
      }
   }
}


proc make-frame {} {
  global env
  global module maxmodule modulelist
  global expertlist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
     :menubutton MODULE-SEL :label "select:" :strings $modulelist

     :hskip 50
     :checkbox ALL-MODULES :label "Act on ALL modules?"

#   :panel SETTINGS  :layout T :padx 10 :pady 10
#     :layout V; :row-gap 10

   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 20; :row-gap 10
      :button REDISPLAY
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software registers]

      set menu {}
      foreach item $expertlist {lappend menu [first $item]}
      :menubutton EXPERT :label "Expert"  :strings $menu

  :show T

}

proc make-electronics {} {
  global env electronicsframe electronicsvisible

 set valuex 32c

  :frame "$electronicsframe" :label "CAEN V1724 Electronics" :resize F :transient T

   :panel P20 :layout T :padx 5 :pady 5
     :layout V; :row-gap 5; :col-gap 10

     :button UPDATE   :label "Electronics Update"   :command redisplay-electronics
     :button SHOWREGS :label "Show register values" :command click-registers

  :show T
 set electronicsvisible 1
}

