#    Diagnostic task for testing VME modules  -   Uses MIDAS web services rather than the register server interface

set frame VMEDIAG
set title "VME Diagnostics V4"
set icon VMEDiag

set started 0
set Testing 0

set VMEAccessServer localhost:8015
set VMEAccessServerHost localhost
set VMEAccessServerPort 8015

package require VMEAccessClient

#    by default rdw and rdl procs write to the log window
set LogFlag 1

set DiagFileBase [file join $env(MIDASBASE) vme tcl diag-scripts]

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set inputprompt  ""
set outputprompt ""
set insertprompt ">>"

set histlim 40
for {set i 0} {$i <= $histlim} {incr i} {lappend history ""}
set nextcommand $histlim

set A16Base 0x0000
set A24Base 0x000000
set A32Base 0x00000000

set AccessMenu [list A16U A16S A24UD A24UP A24SD A24SP A32UD A32UP A32SD A32SP]
set AccessMode A32UD
set WidthMenu [list D8 D16 D32]
set WidthMode D32

set AbortFlag 0


#    procedures taken from the vmecontrol task to obtain list of
#    available VME servers from the experiment file

set RoleList [list VME vme]
set VMECrateList {}

proc make-serverlist {} {
  global VMECrateList RoleList

#    find VME crates available

   set VMECrateList ""
   if {[catch {eg "(experiment ?resources)"} m]} then {
       midas-warning "no experiment connected!!!"
   } else {
       foreach role $RoleList {

          set c [inform ex obtain_SOAPServer_byRole $role]
          foreach item $c {
              append VMECrateList " $item"
          }
       }
   }
#    sort lists into sensible order
   set cratelist [lsort $VMECrateList]
   if {[llength $cratelist] == 0} then {
      midas-warning "no VME crates found in current experiment!!!"
      set VMECrateList {vme}
   } else {
      set VMECrateList {}
      foreach crate $cratelist {lappend VMECrateList $crate}
   }
}

proc click arg {
   global history
   global A16Base A24Base A32Base AccessMenu AccessMode WidthMenu WidthMode LogFlag AbortFlag

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   
   if {$c==":quit"} then {return [click-quit $f]}
   clear-footer
   switch -glob $w  {
   
     A16BASE    {set A16Base $v}
     A24BASE    {set A24Base $v}
     A32BASE    {set A32Base $v}
     
     ACCESS     {set AccessMode [lindex $AccessMenu $v]}
     WIDTH      {set WidthMode [lindex $WidthMenu $v]}
     LOGGING    {set LogFlag $v}
     ABORT      {set AbortFlag 1}
     
     SERVERT    {click-server $v}
     SERVERC    {click-server $i}
     
      CMD        {invoke "$v"}
      HISTORY    {
                 set i [first $v]
                 invoke "[lindex $history $i]"
                 }

      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

#    inital application start
proc start args {
   global started VMECrateList

   if {!$started} {make-initial-frame}

#    built the choice menu for the server
   make-serverlist

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   
   click-server [lindex $VMECrateList 0]
}

proc click-trace {v} {
#    write-register TraceMode $v
#    :modify TRACE :value [read-register TraceMode]
}

proc click-server {v} {
    global VMECrateList MIDASAccessServer
    global VMEAccessServerHost VMEAccessServerPort

    set  MIDASAccessServer $v
    set VMEAccessServerHost [first [split $MIDASAccessServer :]]
    set VMEAccessServerPort [second [split $MIDASAccessServer :]]

    :modify SERVERT :value $MIDASAccessServer
    :modify SERVERC :value [lsearch $VMECrateList $MIDASAccessServer]

    set-footer "Connecting to server $MIDASAccessServer"

    SOAP::configure VMEAccessClient__InterfaceStatus -proxy "http://$VMEAccessServerHost:$VMEAccessServerPort/VMEAccessServer"

    set rc [VMEAccessClient__InterfaceStatus]

    clear-footer 

    if {[first $rc] == 0} {

        SOAP::configure VMEAccessClient__BusRead -proxy "http://$VMEAccessServerHost:$VMEAccessServerPort/VMEAccessServer"
        SOAP::configure VMEAccessClient__BusWrite -proxy "http://$VMEAccessServerHost:$VMEAccessServerPort/VMEAccessServer"

        log "Connected to server $MIDASAccessServer"
        log "Commands are  'rd address'  and  'wr address data'"
    } else {
        log "Cannot connect to server $MIDASAccessServer\n[second $rc]"
    }
}

proc make-initial-frame {} {

    :frame [self frame-name] :label "[self frame-title]" :icon-name "[self icon-title]"
     :message MIDASSORT 
}

proc make-frame {} {
   global env VMECrateList
   global A16Base A24Base A32Base AccessMenu AccessMode WidthMenu WidthMode
   global AbortFlag LogFlag

  set rows 8
  set cols 50

  :set autoreturn "F"

  :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize T

   :panel P1 :layout T :padx 10 :pady 5
     :layout H; :col-gap 5; :row-gap 10
       :text SERVERT         :label "VME Server"  :w 16
       :choice-stack SERVERC :label "" :menu $VMECrateList
#       :hskip 50
#       :checkbox TRACE :label "VME crate TraceMode" 
     :next-row T
     :layout V
       :text A16BASE  :label "A16 base address"  :w 12  :valuex 24c :value $A16Base
       :text A24BASE  :label "A24 base address"  :w 12  :valuex 24c :value $A24Base
       :text A32BASE  :label "A32 base address"  :w 12  :valuex 24c :value $A32Base
       :choice-stack ACCESS :label "VME Address Modifier" :strings $AccessMenu :value [lsearch $AccessMenu $AccessMode]
      :layout H
       :choice-stack WIDTH  :label "VME Data Width"       :strings $WidthMenu  :value [lsearch $WidthMenu $WidthMode]
      :next-row T
       :col-gap 10
       :button ABORT :label "Abort Command"
       :checkbox LOGGING :label "Enable Log" :value $LogFlag
       
   :panel P3 :layout T :padx 10
     :galley LOG :rows $rows :cols $cols :font variable
     configuretags

   :panel P4 :layout T :padx 10 :pady 10
     :layout H; :col-gap 50; :row-gap 10
     :button BAT :label "Execute file"
     :menubutton HISTORY   :label "History"  :strings [list ""]
     :button EMPTYLOG :label "Empty program log window" :command ":modify LOG :empty T"
     :next-row T
     :colour local entry foreground black
     :text CMD  :label "command:" :w [expr $cols - 10]
    bind [:path CMD].entry <Meta-k>         "restorecommand last"
    bind [:path CMD].entry <Meta-BackSpace> "zapcommand"
    bind [:path CMD].entry <Alt-KeyPress-k>         "restorecommand last"
    bind [:path CMD].entry <Alt-KeyPress-BackSpace> "zapcommand"
    bind [:path CMD].entry <Up> "restorecommand previous"
    bind [:path CMD].entry <Down> "restorecommand next"

     open-frame

     :galley_size LOG $rows $cols

    focus [:path CMD].entry
}

proc click-configure {W} {:galley_resize LOG}

proc restorecommand {c} {
  global history histlim nextcommand 
  switch $c {
    last  {:modify CMD :value [lindex $history end]}
    previous  {
     :modify CMD :value [lindex $history $nextcommand]
     incr nextcommand -1
     if {$nextcommand < 0} then {set nextcommand 0}
    }
    next  {
     incr nextcommand 1
     if {$nextcommand > $histlim} then {set nextcommand $histlim}
     :modify CMD :value [lindex $history $nextcommand]
    }
    default {log "restorecommand $c?"}
  }
}

proc zapcommand {} {:modify CMD :value ""}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc click-bat args {
    global env MIDASAccessServer DiagFileBase

#    Here use the built in browser function
#    This uses the canvas function and requires a pale coloured background

    option add *Canvas.background  #c0c020
    set types {
      {{Command Files}      {.bat}   }
      {{All Files}   *        }
    }
    set batfile [tk_getOpenFile -parent [self frame-name] -title "Diagnostic script Selection"  \
       -initialdir $DiagFileBase -filetypes $types]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$batfile != ""} then {
       set directory [file dirname $batfile]
       set file [file tail $batfile]
       set z [midas-confirmation "Execute file $file for the VME card in VME crate $MIDASAccessServer"]
       if {$z == 0} then {return}
       uplevel #0 source $batfile
       set DiagFileBase $directory
    }
}


proc invoke {text} {
    global outputprompt inputprompt
    global history histmenu histlim nextcommand
    global AbortFlag LogFlag

    set AbortFlag 0

    set cmd ""
    foreach item [split $text ", "] {
        if {$item != ""} then {append cmd $item " "}
    }

    :modify LOG :usingtag taginput :strings "[set inputprompt]$cmd\n"

  if [info complete $cmd] {
       if {$cmd == "!!"} {
           set cmd [lindex $history end]
       } else {
           lappend history "$cmd"
           set history [lreplace $history 0 0]
       }
  
    if {[first $cmd] == "repeat"} then {
        set count [second $cmd]
        set l [string length "repeat $count"]
        set cmd [string range $cmd [expr $l+1] end]
    } else {set count 1}

    if {$count == 0} then {set count 100000}
    for {set i 1} {$i <= $count} {incr i} {
       update
       if {$AbortFlag == 1} {:modify LOG :strings "Abort\n"; break}
       set result [catch {eval $cmd} msg]
       if {$result != 0} {
            :modify LOG :usingtag tagoutput :strings "[set outputprompt]Error: $msg\n"
            if {$i <= $count} then {:modify LOG :usingtag tagoutput :strings "loop aborted due to error\n"}
            break;
       } else {
           if {$msg != ""} then {if {$LogFlag == 1 || $i == 1} then {:modify LOG :usingtag tagoutput :strings "$outputprompt$msg\n"}}
       }
    }
  }

    :modify CMD :value ""
    set nextcommand $histlim

    set histmenu {}
    set i 0
    foreach s $history {
       if {$s != ""} then {lappend histmenu "$i [string range "$s" 0 40]"}
       incr i
    }
    :modify HISTORY :strings $histmenu
}

proc log {s} {
  global insertprompt
  :modify LOG :usingtag taginsert :strings "$insertprompt$s\n"
}

#    bus access procedures

proc rd {address} {
    global A16Base A24Base A32Base
    global AccessMode WidthMode
    global LogFlag

   set Base [set [string range $AccessMode 0 2]Base]
   set rc [VMEAccessClient__BusRead [expr $Base + $address] $AccessMode $WidthMode]

   if {$LogFlag == 1} then {log "$AccessMode-$WidthMode: rd [format "0x%04x" [expr $Base + $address]]: $rc"}

   return [second $rc]
}

proc wr {address data} {
    global A16Base A24Base A32Base
    global AccessMode WidthMode
    global LogFlag

   set Base [set [string range $AccessMode 0 2]Base]
   set rc [VMEAccessClient__BusWrite [expr $Base + $address] $AccessMode $WidthMode $data]

   if {$LogFlag == 1} then {log "$AccessMode-$WidthMode: wr [format "0x%04x" [expr $Base + $address]]: $rc"}

   return [second $rc]
}


